@@ -3,5 +3,5 @@
 package cli
 
 const (
-	TERMINAL_CLEAR_LINE = "\r\r"
+	TerminalClearLine = "\r\r"
 )
