@@ -5,11 +5,15 @@ import (
 	"bytes"
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
-	"net"
+	"io"
 	"net/http"
+	"net/url"
+	"os"
 	"regexp"
 	"strings"
+	"syscall"
 	"text/tabwriter"
 
 	"github.com/OJ/gobuster/v3/libgobuster"
@@ -23,14 +27,14 @@ type GobusterGCS struct {
 	bucketRegex *regexp.Regexp
 }
 
-// NewGobusterGCS creates a new initialized GobusterGCS
-func NewGobusterGCS(globalopts *libgobuster.Options, opts *OptionsGCS) (*GobusterGCS, error) {
+// New creates a new initialized GobusterGCS
+func New(globalopts *libgobuster.Options, opts *OptionsGCS, logger *libgobuster.Logger) (*GobusterGCS, error) {
 	if globalopts == nil {
-		return nil, fmt.Errorf("please provide valid global options")
+		return nil, errors.New("please provide valid global options")
 	}
 
 	if opts == nil {
-		return nil, fmt.Errorf("please provide valid plugin options")
+		return nil, errors.New("please provide valid plugin options")
 	}
 
 	g := GobusterGCS{
@@ -54,7 +58,7 @@ func NewGobusterGCS(globalopts *libgobuster.Options, opts *OptionsGCS) (*Gobuste
 		FollowRedirect: true,
 	}
 
-	h, err := libgobuster.NewHTTPClient(&httpOpts)
+	h, err := libgobuster.NewHTTPClient(&httpOpts, logger)
 	if err != nil {
 		return nil, err
 	}
@@ -71,18 +75,30 @@ func (s *GobusterGCS) Name() string {
 }
 
 // PreRun is the pre run implementation of GobusterS3
-func (s *GobusterGCS) PreRun(ctx context.Context, progress *libgobuster.Progress) error {
+func (s *GobusterGCS) PreRun(_ context.Context, _ *libgobuster.Progress) error {
 	return nil
 }
 
 // ProcessWord is the process implementation of GobusterS3
-func (s *GobusterGCS) ProcessWord(ctx context.Context, word string, progress *libgobuster.Progress) error {
+func (s *GobusterGCS) ProcessWord(ctx context.Context, word string, progress *libgobuster.Progress) (libgobuster.Result, error) {
 	// only check for valid bucket names
 	if !s.isValidBucketName(word) {
-		return nil
+		return nil, nil // nolint:nilnil
 	}
 
-	bucketURL := fmt.Sprintf("https://storage.googleapis.com/storage/v1/b/%s/o?maxResults=%d", word, s.options.MaxFilesToList)
+	bucketURL := fmt.Sprintf("https://storage.googleapis.com/storage/v1/b/%s/o?maxResults=%d", url.PathEscape(word), s.options.MaxFilesToList)
+	u, err := url.Parse(bucketURL)
+	if err != nil {
+		return nil, fmt.Errorf("could not parse bucket URL %s: %w", bucketURL, err)
+	}
+
+	// add some debug output
+	if s.globalopts.Debug {
+		progress.MessageChan <- libgobuster.Message{
+			Level:   libgobuster.LevelDebug,
+			Message: fmt.Sprintf("trying word %s", word),
+		}
+	}
 
 	tries := 1
 	if s.options.RetryOnTimeout && s.options.RetryAttempts > 0 {
@@ -94,30 +110,39 @@ func (s *GobusterGCS) ProcessWord(ctx context.Context, word string, progress *li
 	var body []byte
 	for i := 1; i <= tries; i++ {
 		var err error
-		statusCode, _, _, body, err = s.http.Request(ctx, bucketURL, libgobuster.RequestOptions{ReturnBody: true})
+		statusCode, _, _, body, err = s.http.Request(ctx, *u, libgobuster.RequestOptions{ReturnBody: true})
 		if err != nil {
 			// check if it's a timeout and if we should try again and try again
 			// otherwise the timeout error is raised
-			if netErr, ok := err.(net.Error); ok && netErr.Timeout() && i != tries {
+			switch {
+			case os.IsTimeout(err) && i != tries:
 				continue
-			} else if strings.Contains(err.Error(), "invalid control character in URL") {
-				// put error in error chan so it's printed out and ignore it
+			case strings.Contains(err.Error(), "invalid control character in URL"):
+				// put error in error chan, so it's printed out and ignore it
 				// so gobuster will not quit
 				progress.ErrorChan <- err
 				continue
-			} else {
-				return err
+			default:
+				switch {
+				case errors.Is(err, io.EOF):
+					return nil, libgobuster.ErrEOF
+				case os.IsTimeout(err):
+					return nil, libgobuster.ErrTimeout
+				case errors.Is(err, syscall.ECONNREFUSED):
+					return nil, libgobuster.ErrConnectionRefused
+				}
+				return nil, err
 			}
 		}
 		break
 	}
 
 	if statusCode == 0 || body == nil {
-		return nil
+		return nil, nil // nolint:nilnil
 	}
 
 	// looks like 401, 403, and 404 are the only negative status codes
-	found := false
+	var found bool
 	switch statusCode {
 	case http.StatusUnauthorized,
 		http.StatusForbidden,
@@ -134,17 +159,16 @@ func (s *GobusterGCS) ProcessWord(ctx context.Context, word string, progress *li
 	// nothing found, bail out
 	// may add the result later if we want to enable verbose output
 	if !found {
-		return nil
+		return nil, nil // nolint:nilnil
 	}
 
 	extraStr := ""
-	if s.globalopts.Verbose {
+	if s.options.ShowFiles {
 		// get status
 		var result map[string]interface{}
 		err := json.Unmarshal(body, &result)
-
 		if err != nil {
-			return fmt.Errorf("could not parse response json: %w", err)
+			return nil, fmt.Errorf("could not parse response json: %w", err)
 		}
 
 		if _, exist := result["error"]; exist {
@@ -152,7 +176,7 @@ func (s *GobusterGCS) ProcessWord(ctx context.Context, word string, progress *li
 			gcsError := GCSError{}
 			err := json.Unmarshal(body, &gcsError)
 			if err != nil {
-				return fmt.Errorf("could not parse error json: %w", err)
+				return nil, fmt.Errorf("could not parse error json: %w", err)
 			}
 			extraStr = fmt.Sprintf("Error: %s (%d)", gcsError.Error.Message, gcsError.Error.Code)
 		} else if v, exist := result["kind"]; exist && v == "storage#objects" {
@@ -161,7 +185,7 @@ func (s *GobusterGCS) ProcessWord(ctx context.Context, word string, progress *li
 			gcsListing := GCSListing{}
 			err := json.Unmarshal(body, &gcsListing)
 			if err != nil {
-				return fmt.Errorf("could not parse result json: %w", err)
+				return nil, fmt.Errorf("could not parse result json: %w", err)
 			}
 			extraStr = "Bucket Listing enabled: "
 			for _, x := range gcsListing.Items {
@@ -171,16 +195,24 @@ func (s *GobusterGCS) ProcessWord(ctx context.Context, word string, progress *li
 		}
 	}
 
-	progress.ResultChan <- Result{
+	r := Result{
 		Found:      found,
 		BucketName: word,
 		Status:     extraStr,
 	}
 
-	return nil
+	return r, nil
 }
 
-func (s *GobusterGCS) AdditionalWords(word string) []string {
+func (s *GobusterGCS) AdditionalWordsLen() int {
+	return 0
+}
+
+func (s *GobusterGCS) AdditionalWords(_ string) []string {
+	return []string{}
+}
+
+func (s *GobusterGCS) AdditionalSuccessWords(_ string) []string {
 	return []string{}
 }
 
@@ -227,12 +259,18 @@ func (s *GobusterGCS) GetConfigString() (string, error) {
 		}
 	}
 
+	if o.LocalAddr != nil {
+		if _, err := fmt.Fprintf(tw, "[+] Local IP:\t%s\n", o.LocalAddr); err != nil {
+			return "", err
+		}
+	}
+
 	if _, err := fmt.Fprintf(tw, "[+] Timeout:\t%s\n", o.Timeout.String()); err != nil {
 		return "", err
 	}
 
-	if s.globalopts.Verbose {
-		if _, err := fmt.Fprintf(tw, "[+] Verbose:\ttrue\n"); err != nil {
+	if s.options.ShowFiles {
+		if _, err := fmt.Fprintf(tw, "[+] Show Files:\ttrue\n"); err != nil {
 			return "", err
 		}
 	}
