@@ -5,11 +5,15 @@ import (
 	"bytes"
 	"context"
 	"encoding/xml"
+	"errors"
 	"fmt"
-	"net"
+	"io"
 	"net/http"
+	"net/url"
+	"os"
 	"regexp"
 	"strings"
+	"syscall"
 	"text/tabwriter"
 
 	"github.com/OJ/gobuster/v3/libgobuster"
@@ -23,14 +27,14 @@ type GobusterS3 struct {
 	bucketRegex *regexp.Regexp
 }
 
-// NewGobusterS3 creates a new initialized GobusterS3
-func NewGobusterS3(globalopts *libgobuster.Options, opts *OptionsS3) (*GobusterS3, error) {
+// New creates a new initialized GobusterS3
+func New(globalopts *libgobuster.Options, opts *OptionsS3, logger *libgobuster.Logger) (*GobusterS3, error) {
 	if globalopts == nil {
-		return nil, fmt.Errorf("please provide valid global options")
+		return nil, errors.New("please provide valid global options")
 	}
 
 	if opts == nil {
-		return nil, fmt.Errorf("please provide valid plugin options")
+		return nil, errors.New("please provide valid plugin options")
 	}
 
 	g := GobusterS3{
@@ -54,7 +58,7 @@ func NewGobusterS3(globalopts *libgobuster.Options, opts *OptionsS3) (*GobusterS
 		FollowRedirect: true,
 	}
 
-	h, err := libgobuster.NewHTTPClient(&httpOpts)
+	h, err := libgobuster.NewHTTPClient(&httpOpts, logger)
 	if err != nil {
 		return nil, err
 	}
@@ -70,18 +74,30 @@ func (s *GobusterS3) Name() string {
 }
 
 // PreRun is the pre run implementation of GobusterS3
-func (s *GobusterS3) PreRun(ctx context.Context, progress *libgobuster.Progress) error {
+func (s *GobusterS3) PreRun(_ context.Context, _ *libgobuster.Progress) error {
 	return nil
 }
 
 // ProcessWord is the process implementation of GobusterS3
-func (s *GobusterS3) ProcessWord(ctx context.Context, word string, progress *libgobuster.Progress) error {
+func (s *GobusterS3) ProcessWord(ctx context.Context, word string, progress *libgobuster.Progress) (libgobuster.Result, error) {
 	// only check for valid bucket names
 	if !s.isValidBucketName(word) {
-		return nil
+		return nil, nil // nolint:nilnil
 	}
 
-	bucketURL := fmt.Sprintf("https://%s.s3.amazonaws.com/?max-keys=%d", word, s.options.MaxFilesToList)
+	bucketURL := fmt.Sprintf("https://%s.s3.amazonaws.com/?max-keys=%d", url.PathEscape(word), s.options.MaxFilesToList)
+	u, err := url.Parse(bucketURL)
+	if err != nil {
+		return nil, fmt.Errorf("could not parse bucket URL %s: %w", bucketURL, err)
+	}
+
+	// add some debug output
+	if s.globalopts.Debug {
+		progress.MessageChan <- libgobuster.Message{
+			Level:   libgobuster.LevelDebug,
+			Message: fmt.Sprintf("trying word %s", word),
+		}
+	}
 
 	tries := 1
 	if s.options.RetryOnTimeout && s.options.RetryAttempts > 0 {
@@ -93,26 +109,34 @@ func (s *GobusterS3) ProcessWord(ctx context.Context, word string, progress *lib
 	var body []byte
 	for i := 1; i <= tries; i++ {
 		var err error
-		statusCode, _, _, body, err = s.http.Request(ctx, bucketURL, libgobuster.RequestOptions{ReturnBody: true})
+		statusCode, _, _, body, err = s.http.Request(ctx, *u, libgobuster.RequestOptions{ReturnBody: true})
 		if err != nil {
 			// check if it's a timeout and if we should try again and try again
 			// otherwise the timeout error is raised
-			if netErr, ok := err.(net.Error); ok && netErr.Timeout() && i != tries {
+			if os.IsTimeout(err) && i != tries {
 				continue
 			} else if strings.Contains(err.Error(), "invalid control character in URL") {
-				// put error in error chan so it's printed out and ignore it
+				// put error in error chan, so it's printed out and ignore it
 				// so gobuster will not quit
 				progress.ErrorChan <- err
 				continue
-			} else {
-				return err
 			}
+
+			switch {
+			case errors.Is(err, io.EOF):
+				return nil, libgobuster.ErrEOF
+			case os.IsTimeout(err):
+				return nil, libgobuster.ErrTimeout
+			case errors.Is(err, syscall.ECONNREFUSED):
+				return nil, libgobuster.ErrConnectionRefused
+			}
+			return nil, err
 		}
 		break
 	}
 
 	if statusCode == 0 || body == nil {
-		return nil
+		return nil, nil // nolint:nilnil
 	}
 
 	// looks like 404 and 400 are the only negative status codes
@@ -133,17 +157,17 @@ func (s *GobusterS3) ProcessWord(ctx context.Context, word string, progress *lib
 	// nothing found, bail out
 	// may add the result later if we want to enable verbose output
 	if !found {
-		return nil
+		return nil, nil // nolint:nilnil
 	}
 
 	extraStr := ""
-	if s.globalopts.Verbose {
+	if s.options.ShowFiles {
 		// get status
 		if bytes.Contains(body, []byte("<Error>")) {
 			awsError := AWSError{}
 			err := xml.Unmarshal(body, &awsError)
 			if err != nil {
-				return fmt.Errorf("could not parse error xml: %w", err)
+				return nil, fmt.Errorf("could not parse error xml: %w", err)
 			}
 			// https://docs.aws.amazon.com/AmazonS3/latest/API/ErrorResponses.html#ErrorCodeList
 			extraStr = fmt.Sprintf("Error: %s (%s)", awsError.Message, awsError.Code)
@@ -152,7 +176,7 @@ func (s *GobusterS3) ProcessWord(ctx context.Context, word string, progress *lib
 			awsListing := AWSListing{}
 			err := xml.Unmarshal(body, &awsListing)
 			if err != nil {
-				return fmt.Errorf("could not parse result xml: %w", err)
+				return nil, fmt.Errorf("could not parse result xml: %w", err)
 			}
 			extraStr = "Bucket Listing enabled: "
 			for _, x := range awsListing.Contents {
@@ -162,16 +186,24 @@ func (s *GobusterS3) ProcessWord(ctx context.Context, word string, progress *lib
 		}
 	}
 
-	progress.ResultChan <- Result{
+	r := Result{
 		Found:      found,
 		BucketName: word,
 		Status:     extraStr,
 	}
 
-	return nil
+	return r, nil
 }
 
-func (d *GobusterS3) AdditionalWords(word string) []string {
+func (s *GobusterS3) AdditionalWordsLen() int {
+	return 0
+}
+
+func (s *GobusterS3) AdditionalWords(_ string) []string {
+	return []string{}
+}
+
+func (s *GobusterS3) AdditionalSuccessWords(_ string) []string {
 	return []string{}
 }
 
@@ -218,12 +250,18 @@ func (s *GobusterS3) GetConfigString() (string, error) {
 		}
 	}
 
+	if o.LocalAddr != nil {
+		if _, err := fmt.Fprintf(tw, "[+] Local IP:\t%s\n", o.LocalAddr); err != nil {
+			return "", err
+		}
+	}
+
 	if _, err := fmt.Fprintf(tw, "[+] Timeout:\t%s\n", o.Timeout.String()); err != nil {
 		return "", err
 	}
 
-	if s.globalopts.Verbose {
-		if _, err := fmt.Fprintf(tw, "[+] Verbose:\ttrue\n"); err != nil {
+	if s.options.ShowFiles {
+		if _, err := fmt.Fprintf(tw, "[+] Show Files:\ttrue\n"); err != nil {
 			return "", err
 		}
 	}
